package com.link.core.mq.cmq.Tools;

import com.link.core.mq.cmq.Tools.Json.JSONArray;
import com.link.core.mq.cmq.Tools.Json.JSONObject;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

public class Queue {
   protected String queueName;
   protected CMQClient client;

   Queue(String queueName, CMQClient client) {
      this.queueName = queueName;
      this.client = client;
   }

   public QueueMeta getQueueAttributes() throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("queueName", this.queueName);
      String result = this.client.call("GetQueueAttributes", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         QueueMeta meta = new QueueMeta();
         meta.setMaxMsgHeapNum(jsonObj.getInt("maxMsgHeapNum"));
         meta.setPollingWaitSeconds(jsonObj.getInt("pollingWaitSeconds"));
         meta.setVisibilityTimeout(jsonObj.getInt("visibilityTimeout"));
         meta.setMaxMsgSize(jsonObj.getInt("maxMsgSize"));
         meta.setMsgRetentionSeconds(jsonObj.getInt("msgRetentionSeconds"));
         meta.setCreateTime(jsonObj.getInt("createTime"));
         meta.setLastModifyTime(jsonObj.getInt("lastModifyTime"));
         meta.setActiveMsgNum(jsonObj.getInt("activeMsgNum"));
         meta.setInactiveMsgNum(jsonObj.getInt("inactiveMsgNum"));
         meta.setRewindmsgNum(jsonObj.getInt("rewindMsgNum"));
         meta.setMinMsgTime(jsonObj.getInt("minMsgTime"));
         meta.setDelayMsgNum(jsonObj.getInt("delayMsgNum"));
         return meta;
      }
   }

   public void setQueueAttributes(QueueMeta meta) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("queueName", this.queueName);
      if (meta.getMaxMsgHeapNum() > 0) {
         param.put("maxMsgHeapNum", Integer.toString(meta.getMaxMsgHeapNum()));
      }

      if (meta.getPollingWaitSeconds() > 0) {
         param.put("pollingWaitSeconds", Integer.toString(meta.getPollingWaitSeconds()));
      }

      if (meta.getVisibilityTimeout() > 0) {
         param.put("visibilityTimeout", Integer.toString(meta.getVisibilityTimeout()));
      }

      if (meta.getMaxMsgSize() > 0) {
         param.put("maxMsgSize", Integer.toString(meta.getMaxMsgSize()));
      }

      if (meta.getMsgRetentionSeconds() > 0) {
         param.put("msgRetentionSeconds", Integer.toString(meta.getMsgRetentionSeconds()));
      }

      String result = this.client.call("SetQueueAttributes", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      }
   }

   public String sendMessage(String msgBody) throws Exception {
      return this.sendMessage(msgBody, 0);
   }

   public String sendMessage(String msgBody, int delayTime) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("queueName", this.queueName);
      param.put("msgBody", msgBody);
      param.put("delaySeconds", Integer.toString(delayTime));
      String result = this.client.call("SendMessage", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         return jsonObj.getString("msgId");
      }
   }

   public List<String> batchSendMessage(List<String> vtMsgBody) throws Exception {
      return this.batchSendMessage(vtMsgBody, 0);
   }

   public List<String> batchSendMessage(List<String> vtMsgBody, int delayTime) throws Exception {
      if (!vtMsgBody.isEmpty() && vtMsgBody.size() <= 16) {
         TreeMap<String, String> param = new TreeMap();
         param.put("queueName", this.queueName);

         for(int i = 0; i < vtMsgBody.size(); ++i) {
            String k = "msgBody." + Integer.toString(i + 1);
            param.put(k, vtMsgBody.get(i));
         }

         param.put("delaySeconds", Integer.toString(delayTime));
         String result = this.client.call("BatchSendMessage", param);
         JSONObject jsonObj = new JSONObject(result);
         int code = jsonObj.getInt("code");
         if (code != 0) {
            throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
         } else {
            ArrayList<String> vtMsgId = new ArrayList();
            JSONArray jsonArray = jsonObj.getJSONArray("msgList");

            for(int i = 0; i < jsonArray.length(); ++i) {
               JSONObject obj = (JSONObject)jsonArray.get(i);
               vtMsgId.add(obj.getString("msgId"));
            }

            return vtMsgId;
         }
      } else {
         throw new CMQClientException("Error: message size is empty or more than 16");
      }
   }

   public Message receiveMessage(int pollingWaitSeconds) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("queueName", this.queueName);
      if (pollingWaitSeconds > 0) {
         param.put("UserpollingWaitSeconds", Integer.toString(pollingWaitSeconds * 1000));
         param.put("pollingWaitSeconds", Integer.toString(pollingWaitSeconds));
      } else {
         param.put("UserpollingWaitSeconds", Integer.toString(30000));
      }

      String result = this.client.call("ReceiveMessage", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         Message msg = new Message();
         msg.setMsgId(jsonObj.getString("msgId"));
         msg.setReceiptHandle(jsonObj.getString("receiptHandle"));
         msg.setMsgBody(jsonObj.getString("msgBody"));
         msg.setEnqueueTime(jsonObj.getLong("enqueueTime"));
         msg.setNextVisibleTime(jsonObj.getLong("nextVisibleTime"));
         msg.setFirstDequeueTime(jsonObj.getLong("firstDequeueTime"));
         msg.setDequeueCount(jsonObj.getInt("dequeueCount"));
         return msg;
      }
   }

   public List<Message> batchReceiveMessage(int numOfMsg, int pollingWaitSeconds) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("queueName", this.queueName);
      param.put("numOfMsg", Integer.toString(numOfMsg));
      if (pollingWaitSeconds > 0) {
         param.put("UserpollingWaitSeconds", Integer.toString(pollingWaitSeconds * 1000));
         param.put("pollingWaitSeconds", Integer.toString(pollingWaitSeconds));
      } else {
         param.put("UserpollingWaitSeconds", Integer.toString(30000));
      }

      String result = this.client.call("BatchReceiveMessage", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         ArrayList<Message> vtMessage = new ArrayList();
         JSONArray jsonArray = jsonObj.getJSONArray("msgInfoList");

         for(int i = 0; i < jsonArray.length(); ++i) {
            JSONObject obj = (JSONObject)jsonArray.get(i);
            Message msg = new Message();
            msg.setMsgId(obj.getString("msgId"));
            msg.setReceiptHandle(obj.getString("receiptHandle"));
            msg.setMsgBody(obj.getString("msgBody"));
            msg.setEnqueueTime(obj.getLong("enqueueTime"));
            msg.setNextVisibleTime(obj.getLong("nextVisibleTime"));
            msg.setFirstDequeueTime(obj.getLong("firstDequeueTime"));
            msg.setDequeueCount(obj.getInt("dequeueCount"));
            vtMessage.add(msg);
         }

         return vtMessage;
      }
   }

   public void deleteMessage(String receiptHandle) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("queueName", this.queueName);
      param.put("receiptHandle", receiptHandle);
      String result = this.client.call("DeleteMessage", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      }
   }

   public void batchDeleteMessage(List<String> vtReceiptHandle) throws Exception {
      if (!vtReceiptHandle.isEmpty()) {
         TreeMap<String, String> param = new TreeMap();
         param.put("queueName", this.queueName);

         for(int i = 0; i < vtReceiptHandle.size(); ++i) {
            String k = "receiptHandle." + Integer.toString(i + 1);
            param.put(k, vtReceiptHandle.get(i));
         }

         String result = this.client.call("BatchDeleteMessage", param);
         JSONObject jsonObj = new JSONObject(result);
         int code = jsonObj.getInt("code");
         if (code != 0) {
            throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
         }
      }
   }
}
